/*************************************************************************************************************************************************/
/*!     DAC161.c
*
*       This code is designed to perform standard command and control operations on the DAC161 over a SPI bus. Functions exist to setup, configure,
*       and control the DAC161.
*
*
*/
/**************************************************************************************************************************************************
*       Copyright  2014 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
***************************************************************************************************************************************************
*  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
*                                                                                                                                                 *
*    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
*                                                                                                                                                 *
*    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
*    documentation and/or other materials provided with the distribution.                                                                         *
*                                                                                                                                                 *
*    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
*    from this software without specific prior written permission.                                                                                *
*                                                                                                                                                 *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
**************************************************************************************************************************************************/
#include <msp430.h>
#include "DAC161.h"
#include "TempTransmitter.h"

static unsigned char RcvData[3];

/*************************************************************************************************************************************************
*  DAC161_Write_Regs
**************************************************************************************************************************************************/
/*!
* @brief Writes registers on the DAC161.
*
* This function will execute a write register command to the DAC161. This function can be used to update one or more registers on the DAC161.
* No error checking is performed, so it is the user's responsibility to make sure they do not attempt to write past the end of the DAC161 registers.
*
* @param[out]   *writeValues    Pointer to the list of 8 bit register values to place in the DAC161
* @param[in]     startReg       Address of the first register to write
* @param[in]     length         Number of registers to write.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Write_Regs (unsigned short *writeValues, unsigned char startReg, unsigned char lengthBytes)
{
    unsigned char outData[3];

    outData[0] = DAC161_SPI_WRITE_CMD(startReg);

    // Switch Endianess
    outData[1] = *writeValues >> 8;
    outData[2] = *writeValues & 0xff;

    USCIA0_SPI_write (outData, RcvData, lengthBytes+1);    // Add 1 to length for command byte

}

/*************************************************************************************************************************************************
*  DAC161_Read_Regs
**************************************************************************************************************************************************/
/*!
* @brief Reads registers on the DAC161.
*
* This function will execute a read register command to the DAC161 and return the resultant data. This function can be used to read one or more
* registers from the DAC161. No error checking is performed, so it is the user's responsibility to make sure they do not attempt to read past
* the end of the DAC161 registers.
*
* @param[out]   *readValues     Pointer to place the 8 bit register values from the DAC161
* @param[in]     startReg       Address of the first register to read
* @param[in]     length         Number of registers to read.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Read_Regs (unsigned char *readValues, unsigned char startReg, unsigned char lengthBytes)
{
    unsigned char outData[3] = {0x55, 0x55, 0x55};

    // Sets the address for the read command
    outData[0] = DAC161_SPI_READ_CMD(startReg);
    USCIA0_SPI_write (outData, readValues, lengthBytes+1);    // Add 1 to length for command byte

    // Performs actual read of previous address
    outData[0] = DAC161_NOP_REG;
    USCIA0_SPI_write (outData, readValues, lengthBytes+1);    // Add 1 to length for command byte

}

/*************************************************************************************************************************************************
*  Setup_DAC161
**************************************************************************************************************************************************/
/*!
* @brief Performs the setup of the DAC161.
*
* This function will configure the DAC161.
*
* @param[in]     errConfig       Error Configuration for the DAC161 (DAC161_STD_ERR_CONFIG)
* @param[in]     errLow_uA       Output level (micro Amps) for a Low Error
* @param[in]     errHigh_uA      Output level (micro Amps) for a High Error
*
* @return  None
*
**************************************************************************************************************************************************/
void Setup_DAC161  (unsigned short errConfig, unsigned short errLow_uA, unsigned short errHigh_uA)
{
    unsigned short errValue;

    DAC161_Write_Regs (&errConfig, DAC161_ERR_CONFIG_REG, 2);

    errValue = DAC161_CONVERT_ERR_VALUE(errLow_uA) & 0x7f00;
    DAC161_Write_Regs (&errValue, DAC161_ERR_LOW_REG, 2);

    errValue = DAC161_CONVERT_ERR_VALUE(errHigh_uA) & 0xff00;
    if (errValue < 0x80)
        errValue = 0x80;
    DAC161_Write_Regs (&errValue, DAC161_ERR_HIGH_REG, 2);

}


/*************************************************************************************************************************************************
*  DAC161_Reset
**************************************************************************************************************************************************/
/*!
* @brief Sends a Reset Command to the DAC161.
*
* This function sends a Reset command to the DAC161 on the SPI bus.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Reset (void)
{
    unsigned short resetCode = DAC161_RESET_VALUE;

    DAC161_Write_Regs (&resetCode, DAC161_RESET_REG, 2);
    DAC161_Write_Regs (&resetCode, DAC161_NOP_REG, 2);          // Nop required after reset

}

/*************************************************************************************************************************************************
*  DAC161_Nop
**************************************************************************************************************************************************/
/*!
* @brief Sends a Nop Command to the DAC161.
*
* This function sends a Nop command to the DAC161 on the SPI bus. The DAC161 will timeout and move into an error state if it does not receive
* regular commands for the SPI master. This command can be used to notify the DAC161 that the system is still operational, but no change to the
* DAC161 is desired.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Nop (void)
{
    unsigned short deadCode = 0xdead;

    DAC161_Write_Regs (&deadCode, DAC161_NOP_REG, 2);

}

/*************************************************************************************************************************************************
*  DAC161_Set_Out_Value
**************************************************************************************************************************************************/
/*!
* @brief Sets the output current from the DAC161.
*
* The DAC161 is designed to be used in a 4-20mAmp system. Data is communicated by changing the current output from the DAC161. This function sets
* the desired output current.
*
* @param[in]     uAmps         Value in uAmps to output from the DAC161
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Set_Out_Value (unsigned long uAmps)
{
    unsigned short value = DAC161_OUTPUT_CONVERT(uAmps);

    DAC161_Write_Regs (&value, DAC161_DACCODE_REG, 2);

}

/*************************************************************************************************************************************************
*  DAC161_Read_Status
**************************************************************************************************************************************************/
/*!
* @brief Reads thE status register from the DAC161.
*
* This function returns the current value in the status register of the DAC161
*
* @return  DAC161 Status     (DAC161_FERR_STS, DAC161_SPI_TIMEOUT_ERR, DAC161_LOOP_STS, DAC161_CURR_LOOP_STS)
**************************************************************************************************************************************************/
unsigned char DAC161_Read_Status (void)
{
    unsigned char returnValue[3];

    DAC161_Read_Regs (returnValue, DAC161_STATUS_REG, 2);

    return (returnValue[2]);
}

/*************************************************************************************************************************************************
*  Setup_USCIB0_SPI
**************************************************************************************************************************************************/
void USCIA0_setupMasterSPI(void)
{

    P2SEL1  |= USCIA0_SIMO + USCIA0_SOMI;
    P1SEL1  |= USCIA0_CLK;                                                     // Assign SPI pins to Primary Peripheral of USCI_B0
    DAC_CS_PORTOUT |= DAC_CS_N;
    DAC_CS_PORTDIR |= DAC_CS_N;

    UCA0CTL1  |= UCSWRST;                                                      // Enable SW reset
    UCA0CTLW0 = (UCMST + UCSSEL_2 + UCMSB + UCSYNC + UCCKPH + UCSWRST);                   // SPI Master, 3 wire, synchronous mode
    UCA0BR0 = 8;                                                               // SMCLK/8 = SCLK (1MHz)
    UCA0BR1 = 0;
    UCA0CTL1 &= ~UCSWRST;                                                      // Clear SW reset, resume operation
}


/*************************************************************************************************************************************************
*  SPI_Write
**************************************************************************************************************************************************/
/*!
* @brief Perform a SPI communication sequence to Write/Read data on SPI bus
*
* The SPI bus uses one clock line and 2 data lines that transfer data at the same time. The master will read data from the MISO
* (Master In Slave Out) line on one edge of the clock. The slave device will read on the MOSI (Master Out Slave In) on the other edge of the clock.
* As such, reads and writes happen together. If there is no data to send to the slave, the master must write out dummy data on the bus to get the
* clock to toggle for the slave device to communicate.
*
* Note that many SPI devices do not communicate back to the master at all, and have no MISO line connected.
*
* @param[in]   outData              Pointer to an array with the data that will be written on the SPI bus
* @param[in]   inData               Pointer to an array to place the data returned from the slave device. One byte for EVERY byte sent,
*                                       even the command byte
* @param[in]   length               Number of bytes to send/receive
*
**************************************************************************************************************************************************/
void USCIA0_SPI_write (unsigned char *outData, unsigned char *inData, unsigned char length)
{
unsigned char i;

    // CS Active Low
    DAC_CS_PORTOUT &= ~DAC_CS_N;
    __delay_cycles(100);          // Pause a bit

    for (i=0; i<length; i++)
    {
        inData[i] = USCIA0_SPI_writeByte (outData[i]);
    }
    while (!(UCA0IFG&UCTXIFG));               // USCI_B0 TX buffer ready?
    __delay_cycles(100);          // Pause a bit
    DAC_CS_PORTOUT |= DAC_CS_N;

}

/*************************************************************************************************************************************************
*  SPI_Write_Byte
**************************************************************************************************************************************************/
/*!
* @brief This function is used to place a single byte on the SPI bus. This function is for expert mode only.
*
* This function can be used by someone managing their own communications on the SPI bus to place one byte on the bus. It is assumed that the user
* will manage the STE pin themselves when using this command. The received byte from the MISO pin will be returned by this function.
*
* This function only exists to support users who have non-standard SPI devices on their bus. Some devices require excessive time between bytes or
* a delay between the command byte and subsequent data bytes.
*
* @param[in]   data              Pointer to an array with the data that will be written on the SPI bus
*
* @return  in_data               Character sent from SPI device on MISO line
**************************************************************************************************************************************************/
unsigned char USCIA0_SPI_writeByte (unsigned char data)
{
    while (!(UCA0IFG&UCTXIFG));               // USCI_B0 TX buffer ready?
    UCA0IFG &= ~UCRXIFG;             // Clear RX Interrupt flag

    UCA0TXBUF = data;              // Place data in TX Buffer
    while (!(UCA0IFG & UCRXIFG));    // Wait for end of data receive
    return ((unsigned char)UCA0RXBUF);            // Return the received byte from RX Buffer
}
