/*************************************************************************************************************************************************/
/*!      Small Form Factor 4-20mA Temperature Transmitter
 *
 *       This code is designed to implement a Temperature Transmitter application highlighting the ADS1220 to receive data from an RTD Temperature
 *       probe and send out the temperature reading on a 4-20mA signal using the DAC161S997.
 *
 *
 */
/**************************************************************************************************************************************************
 *       Copyright  2014 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
 ***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
 *                                                                                                                                                 *
 *    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
 *                                                                                                                                                 *
 *    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
 *    documentation and/or other materials provided with the distribution.                                                                         *
 *                                                                                                                                                 *
 *    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
 *    from this software without specific prior written permission.                                                                                *
 *                                                                                                                                                 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
 **************************************************************************************************************************************************/
//
//   TIDA-00165 v1.0
//
//   Texas Instruments Inc.
//   May 2014
//   Built with Code Composer Studio V6.0
//*************************************************************************************************************************************************


#include <msp430.h>
#include "TempTransmitter.h"
#include "ADS1220.h"
#include "DAC161.h"
#include "RTD_Math.h"


// Setting the DacErrorCorrection to 1.0 and turning on the DAC_TEST will allow the user to measure the
//  output of the DAC161 at 2mA intervals to determine the DacErrorCorrection for the specific device.
//#define DAC_TEST

#define SAMPLES_PER_DAC_ADJUSTMENT       16

const float DacErrorCorrection = 1.0;

// The ADS1220 Gain Calibration is intended to be performed once for any system. The results will be stored in the
//  FRAM memory. To perform GainCalibration, a known precision resistor is placed into the spot for the RTD
//  and a series of measurements are taken. The measured value is compared to the expected result in the
//  MeasuredGainCodeValue variable below. The GainCalibration value is then calculated and stored in memory.

// To prevent the Calibration from being overwritten, the calibration only occurs when the user hits the
// top button AND the value in the FRAM memory is the erased memory value of 0xffffffff. If the user wishes to
// recompute the calibration, the user must change the
// value in the FRAM memory to 0xffffffff.

// To calculate expected result, take the expected answer from the precision multimeter reading and
//   Code = Vrtd  PGA * (2^23 - 1) / Vref
//   Code = (Rmeas  IIDAC1  PGA) * (2^23 - 1) / (2  IDAC1  Rref)
//   Code = Rmeas * PGA * (2^23 - 1) / (Rref * 2)
//   Rref = 3.24k

const float MeasuredGainCodeValue = 7868944.1883;   // for 380 Ohm resistor
const float Rref = 3240.0;                          // reference resistor value on the board is 3.24k Ohm

#pragma PERSISTENT(ADS1220GainCorrection)
float ADS1220GainCorrection = 1.0;            // Gain correction factor; needs to be calculated once for any system

#pragma PERSISTENT(dacStatus)
unsigned char dacStatus = 0;

unsigned char StartConversion;
unsigned char ReadConversionData = 0;
unsigned char DAC_Error_Processing;
unsigned char GainCal = 0;

#ifdef DAC_TEST
unsigned char DacIncrement = 1;              // Testing variable to allow the button to enable/disable DAC processing
#endif

extern long rawData;

void Setup_IO (void);
unsigned long ConvertTemp2uAmps (long temperature);

void main (void)
{
    volatile static unsigned char tempData[3];
    unsigned long dacValueForTemp_uA = 0;
    unsigned long uncorrectedDacValue;
    unsigned char calibrateCount = 0;
    long          temperature;
    long          code;
    float avgTemp;
    float avgRaw;
    unsigned char avgCount = 0;

    WDTCTL = WDTPW | WDTHOLD;   // Stop watchdog timer

    Setup_IO ();

    __enable_interrupt();

    // Determine the ADS1220 Calibration offset - Short the AIN0 and AIN1 together and measure the results 4 times.
    // The average result will be subtracted from all future measurements.
    Setup_ADS1220 (ADS1220_MUX_SHORTED, ADS1220_OP_MODE_NORMAL,
            ADS1220_CONVERSION_SINGLE_SHOT, ADS1220_DATA_RATE_20SPS, ADS1220_GAIN_16, ADS1220_USE_PGA,
            ADS1220_IDAC1_AIN3, ADS1220_IDAC2_AIN2, ADS1220_IDAC_CURRENT_250_UA);

    // Configure the DAC161
    Setup_DAC161  (DAC161_STD_ERR_CONFIG_MASKED, 3370, 21800);
    DAC_Error_Processing = 0;

    // Delay 500ms to allow for IDAC and Input Filter settling
    __delay_cycles(500000);

    ReadConversionData = 0;
    ADS1220_Start ();             // Kick off conversion

    // Gather and average 8 readings from the ADS1220
    while (calibrateCount < 8)
    {
        while (!ReadConversionData);   // Wait for Data Ready interrupt
        ReadConversionData = 0;
        ADS1220_Get_Conversion_Data ((unsigned char *)tempData);   // Get the raw data
        ADS1220_Offset_Calibrate_Data ((unsigned char *)tempData);        // Send results to calibration function
        calibrateCount++;

        // Start next calibration reading?
        if (calibrateCount < 8)
            ADS1220_Start ();
    }

    // Configure ADS1220 for actual measurements
    Setup_ADS1220 (ADS1220_MUX_AIN0_AIN1, ADS1220_OP_MODE_NORMAL,
            ADS1220_CONVERSION_CONTINUOUS, ADS1220_DATA_RATE_20SPS, ADS1220_GAIN_16, ADS1220_USE_PGA,
            ADS1220_IDAC1_AIN3, ADS1220_IDAC2_AIN2, ADS1220_IDAC_CURRENT_250_UA);

    StartConversion = 0;
    ReadConversionData = 0;

    ADS1220_Start ();      // Only one start needed for Continuous Mode
    DAC161_Set_Out_Value (dacValueForTemp_uA);

    while (1)
    {
        __bis_SR_register(CPUOFF);

        // In case we extend the time between samples in the future, we will still need to keep the DAC
        // from timing out by sending a NOP command every time the loop is processed.
        DAC161_Nop();

        // Every time the Data Ready interrupt occurs, the Conversion Data is read
        if (ReadConversionData)
        {
            StartConversion = 1;             // Start next conversion immediately
            ADS1220_Get_Conversion_Data_Calibrated ((unsigned char *)tempData);   // Get the temperature data with the calibration value subtracted

            code = (((long)tempData[0] << 16) + ((long)tempData[1] << 8) + (long)tempData[2]);   // take 3 bytes of voltage returned and place in word
            // In case we extend the time between samples in the future, we will still need to keep the DAC
            // from timing out by sending a NOP command every time the loop is processed.
            DAC161_Nop();

            temperature = (long)(interpolateTemperatureValue (code) * 1000.0);    // Get the temperature and convert into millidegrees C

            ReadConversionData = 0;                        // Clear flag so next conversion is read again.

            // Collect the measured values for SAMPLES_PER_DAC_ADJUSTMENT(16) measurements to average
            if (avgCount == 0)
            {
                // Erase all previous value, as this is the first measurement for this average block
                avgTemp = (float)temperature;
                avgRaw = (float)rawData;
            }
            else
            {
                // Add values from this measurement into the average block
                avgTemp += (float)temperature;
                avgRaw += (float)rawData;

            }
            avgCount ++;

            // If this is the last measurement, calculate the averages and set the system to print
            if (avgCount == SAMPLES_PER_DAC_ADJUSTMENT)
            {
                avgCount = 0;
                temperature = (long)(avgTemp / SAMPLES_PER_DAC_ADJUSTMENT);
                rawData = (long)(avgRaw / SAMPLES_PER_DAC_ADJUSTMENT);
                //CalVoltage = (long)(avgOffset / SAMPLES_PER_DAC_ADJUSTMENT);
                //GainVoltage = (long)(avgGain / SAMPLES_PER_DAC_ADJUSTMENT);
                //GainCalibrateValue = (long)(avgGainCal / SAMPLES_PER_DAC_ADJUSTMENT);

                // Perform Gain Calibration when the user pushes the button on the top of the device.
                // The user MUST have modified the code to place the expected results into the MeasuredGainCodeValue
                //  variable at the top of main.c. The user is expected to place a known resistor value in place of the
                //  RTD and perform gain calibration to determine the system level gain correction. This value will be
                //  stored into flash memory for future use.
                if (GainCal)
                {
                    GainCal = 0;
                    ADS1220GainCorrection = MeasuredGainCodeValue / rawData;
                }

#ifndef DAC_TEST   // Remove for test code

                // Convert the average temperature into a DAC uAmp value  (4000uA = -200 C;   20000uA = +850 C;
                // We will take the measured error of the DAC161 into account by multiplying by a DacErrorCorrection factor.
                // For this application, we have measured a fairly linear error, and therefore determined a single correction
                // factor that we will use for all measurements.
                //  For customers wishing even greater accuracy, the DAC error could be interpolated over multiple points
                //  in the curve, with different correction factors for different areas along the curve.
                uncorrectedDacValue = ConvertTemp2uAmps (temperature);
                dacValueForTemp_uA = (unsigned long)((float)uncorrectedDacValue * DacErrorCorrection);

                // Output value on DAC
                DAC161_Set_Out_Value (dacValueForTemp_uA);
#endif
            }


#ifdef DAC_TEST   // Test code

            // This block is used to test the DAC. Every time the button on the bottom of the device is pressed,
            // the output of the DAC is increased by 2mA, between 4 and 20mA.
            if (DacIncrement)
            {
                float dacValue;

                DacIncrement = 0;
                // Test code - Ramp from 6mA to 18mA and start over again at 6mA.
                dacValueForTemp_uA += 2000;
                if (dacValueForTemp_uA > 20000)
                    dacValueForTemp_uA = 4000;

                // Apply the DacErrorCorrection to the desired value. If this is measuring
                dacValue = (float)dacValueForTemp_uA * DacErrorCorrection;
                DAC161_Set_Out_Value ((unsigned long)dacValue);
            }
#endif
        }

        // Did the DAC161 indicate an error?
        if (DAC_Error_Processing)
        {
            // Perform DAC ERROR Processing Here
            DAC_Error_Processing = 0;
            dacStatus = DAC161_Read_Status();         // Read Status to clear error

        }

    }

}


/*************************************************************************************************************************************************
 *  Setup_IO
 **************************************************************************************************************************************************/
/*!
 * @brief Initialize the GPIO pins on the MSP430G2413 for use by the Temperature Transmitter
 *
 * This function is called once to configure the GPIO pins on the system.
 *
 * @return  none
 **************************************************************************************************************************************************/
void Setup_IO (void)
{
    PAOUT = 0x0000;
    PJOUT = 0x00;

    ADS_CS_PORTOUT |= ADS_CS_N;                                                // Active Low, so set high
    ADS_CS_PORTDIR |= ADS_CS_N;
    DAC_CS_PORTOUT |= DAC_CS_N;                                                // Active Low, so set high
    DAC_CS_PORTDIR |= DAC_CS_N;
    USCIB0_setupMasterSPI();
    USCIA0_setupMasterSPI();

    P1OUT &= ~(ERR_LVL);                    // Initially off
    P1DIR |=  (ERR_LVL);
    P1DIR &= ~(DAC_ERROR_N | ADS_DRDY_N | USER_INTERRUPT_N);   // Input Pins
    P1IES |=  (DAC_ERROR_N | ADS_DRDY_N | USER_INTERRUPT_N);   // High to Low Edge
    P1IFG &= ~0xFF;                                            // Clear any pending interrupts
    P1IE  |=  (DAC_ERROR_N | ADS_DRDY_N | USER_INTERRUPT_N);   // Enable Interrupts

    //unused pins output low
    P1DIR |= 0x0C;
    PJDIR |= 0x1C;

}



/*************************************************************************************************************************************************
 *  ConvertTemp2uAmps
 **************************************************************************************************************************************************/
/*!
 * @brief Converts a temperature in the range of -200 to +850 degrees C into a 4-20mAmp value.
 *
 * This function is called to convert the temperature (scaled into degrees C * 1000) into an Amp setting to send on the 4-20mA line.
 * -200 degrees will correspond to 4.000 mA. +850 degrees C will correspond to 20mA.
 *
 * @param[in]      temperature  - value in degrees C * 1000; from -200000 to +850000
 *
 * @return  uAmp value       Input temperature converted in MicroAmps (mA * 1000).
 **************************************************************************************************************************************************/
unsigned long ConvertTemp2uAmps (long temperature)
{
    float uAmps;

    // Here is where we need to convert the 3 bytes received from the ADS1220 into a uAmp value that will be placed into the DAC
    // -200 to +850 C measured in 1000 * C is  -200000 to 850000 or a range of 1050000 1000*C.
    // Add 200000 to the temperature to convert it into a range between 0 and 1050000. Divide by 1050000 to get the percentage of the temperature scale.
    //
    // 4 to 20mA output is measured in uAmps and therefore is 4000 to 20000 uAms or a range of 16000 uAmps.
    // Multiply the results above by 16000 to get the percentage on the uAmp scale that ranges from 0 - 16000.
    // Adjust the results so that -200 degrees C (which comes out to 0 uAmps) is represented by 4000 uAmps by adding 4000
    //
    // uAmps = ((temperature + 200000) * 16000 / 1050000) + 4000  --  Divide top and bottom by 1000 and we get
    //         ((temperature + 200000) * 16 / 1050) + 4000
    //

    uAmps = (((temperature + 200000) * 16.0) / 1050.0) + 4000.0;
    return ((unsigned long)uAmps);
}


/**************************************************************************************************************************************************
 * PORT1_ISR
 **************************************************************************************************************************************************/
/*!
 * @brief Port 1 Interrupt Routine
 *
 * This interrupt will occur when the system button is pushed.
 *
 * @return None
 **************************************************************************************************************************************************/
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
    // Has a Data Ready occurred from the ADS1220?
    if (P1IFG & ADS_DRDY_N)
    {
        ReadConversionData = 1;
    }
    // Has an error from the DAC occurred?
    if (P1IFG & DAC_ERROR_N)
    {
        DAC_Error_Processing = 1;
    }

    // The user button will turn on/off the UART outputs
    if (P1IFG & USER_INTERRUPT_N)
    {
#ifndef DAC_TEST   // NOT DAC Test code
        // If there is no GainCalibration in place, allow the system to perform GainCalibration
        if (*(unsigned long *)&ADS1220GainCorrection == 0xffffffff)
        GainCal = 1; //allow the system to perform GainCalibration
#else              // DAC Test code - Use user button to increment the DAC output level
        DacIncrement = 1;                 // During testing, we will use the button to increment dac processing. Default OFF
#endif
    }

    P1IFG = 0;
    __bic_SR_register_on_exit(CPUOFF);      // Exit LPM0

}

